@@ -1,6 +1,10 @@
+#' @keywords internal
+#' @aliases alakazam-package
+"_PACKAGE"
+
 # Alakazam package documentation and import directives
 
-#' The alakazam package
+#' The Alakazam package
 #' 
 #' \code{alakazam} in a member of the Immcantation framework of tools and serves five main 
 #' purposes:
@@ -65,6 +69,7 @@
 #'   \item  \link{getAllele}:            Get V(D)J allele names.
 #'   \item  \link{getGene}:              Get V(D)J gene names.
 #'   \item  \link{getFamily}:            Get V(D)J family names.
+#'   \item  \link{junctionAlignment}: Junction alignment properties 
 #' }
 #' 
 #' @section  Sequence distance calculation:
@@ -109,32 +114,28 @@
 #' @import      methods
 #' @import      utils
 #' @importFrom  airr        read_rearrangement write_rearrangement
-#' @importFrom	ape 		read.tree di2multi reorder.phylo root ladderize
-#' @importFrom  Biostrings  BString extractAt
+#' @importFrom	ape 		read.fastq read.tree di2multi reorder.phylo root ladderize
 #' @importFrom  dplyr       do n desc %>%
 #'                          bind_cols bind_rows combine arrange left_join
 #'                          group_by ungroup
 #'                          filter slice select 
 #'                          mutate mutate_at 
-#' 							one_of if_else
+#' 							one_of
 #'							right_join rowwise
-#'                          summarize summarize_at
+#'                          summarize summarize_at all_of
 #'                          transmute rename
-#' @importFrom  GenomicAlignments explodeCigarOps explodeCigarOpLengths                       
 #' @importFrom  igraph      V E graph_from_data_frame as_data_frame as_edgelist 
 #'                          make_graph make_directed_graph make_undirected_graph
 #'                          vertex_attr set_vertex_attr
 #'                          degree shortest_paths all_shortest_paths distances
 #'                          graph_from_adjacency_matrix components groups
-#' @importFrom  IRanges     IRanges
-#' @importFrom  lazyeval    interp
 #' @importFrom  Matrix      sparseMatrix rowSums
 #' @importFrom  progress    progress_bar
 #' @importFrom  readr       read_delim read_tsv write_delim write_tsv cols
 #' @importFrom  rlang       := sym syms enquo
 #' @importFrom  scales      log2_trans log10_trans trans_breaks trans_format
 #'                          math_format percent scientific pretty_breaks
-#' @importFrom  seqinr      translate
+#' @importFrom  seqinr      translate s2c
 #' @importFrom  stats       na.omit setNames ecdf sd cor cov median mad
 #'                          dbinom pbinom qbinom rbinom
 #'                          dnorm pnorm qnorm rnorm
@@ -149,15 +150,15 @@
 #' @importFrom  tibble      tibble
 #' @importFrom  tidyr       complete gather
 #' @importFrom  Rcpp        evalCpp
+#' @importFrom  Biostrings         BString extractAt
+#' @importFrom  GenomicAlignments  explodeCigarOps explodeCigarOpLengths                       
+#' @importFrom  IRanges            IRanges
 #' @useDynLib   alakazam, .registration=TRUE
 NULL
 
 # Package loading actions
 .onAttach <- function(libname, pkgname) {
-    msg <- paste("As of v1.0.0 the AIRR Rearrangement schema is now the default file format.",
-                 "A description of the standard is available at https://docs.airr-community.org.",
-                 "The legacy Change-O format is supported through arguments to each function",
-                 "that allow the input column names to be explicitly defined.",
-                 sep="\n")
+    msg <- citation(pkgname)
+    msg <-paste(c(format(msg,"citation")),collapse="\n\n")
     packageStartupMessage(msg)
 }
