@@ -155,7 +155,7 @@ countGenes <- function(data, gene, groups=NULL, copy=NULL, clone=NULL, fill=FALS
     }
 
     # Rename gene column
-    gene_tab <- rename(gene_tab, "gene"=gene)
+    gene_tab <- gene_tab %>% rename(all_of(c("gene"=gene)))
     
     return(gene_tab)
 }
@@ -189,7 +189,7 @@ countGenes <- function(data, gene, groups=NULL, copy=NULL, clone=NULL, fill=FALS
 #' @return    A character vector containing allele, gene or family names.
 #' 
 #' @references
-#'   \url{http://imgt.org}
+#'   \url{https://www.imgt.org/}
 #'
 #' @seealso  \link{countGenes}
 #'
@@ -237,7 +237,14 @@ countGenes <- function(data, gene, groups=NULL, copy=NULL, clone=NULL, fill=FALS
 #' getAllele(nl_call, first=FALSE, omit_nl=TRUE)
 #' getGene(nl_call, first=FALSE, omit_nl=TRUE)
 #' getFamily(nl_call, first=FALSE, omit_nl=TRUE)
-#'
+#' 
+#' # Temporary designation examples
+#' tmp_call <- c("IGHV9S3*01", "IGKV10S12*01")
+#' 
+#' getAllele(tmp_call)
+#' getGene(tmp_call)
+#' getFamily(tmp_call)
+#' 
 #' @export
 getSegment <- function(segment_call, segment_regex, first=TRUE, collapse=TRUE, 
                        strip_d=TRUE, omit_nl=FALSE, sep=",") {
@@ -247,11 +254,11 @@ getSegment <- function(segment_call, segment_regex, first=TRUE, collapse=TRUE,
     # Remove NL genes
     if (omit_nl) {
         # Clean segment_call to keep only the name (remove species)
-        allele_regex <- '((IG[HKL][VDJADEGMC]|TR[ABDG])[A-Z0-9\\(\\)]+[-/\\w]*[-\\*]*[\\.\\w]+)'
+        allele_regex <- '((IG[HKL]|TR[ABDG])[VDJADEGMC][A-R0-9\\(\\)]*[-/\\w]*[-\\*]*[\\.\\w]+)'
         segment_call <- gsub(paste0(edge_regex, "(", allele_regex, ")", edge_regex), "\\1", 
                   segment_call, perl=T)
         # non-localized regex
-        nl_regex <- paste0('(IG[HKL][VDJADEGMC]|TR[ABDG])[0-9]+-NL[0-9]([-/\\w]*[-\\*][\\.\\w]+)*(',
+        nl_regex <- paste0('(IG[HKL]|TR[ABDG])[VDJADEGMC][0-9]+-NL[0-9]([-/\\w]*[-\\*][\\.\\w]+)*(',
                            sep, "|$)")
         # delete non-localized calls
         segment_call <- gsub(nl_regex, "", segment_call, perl=TRUE)
@@ -282,7 +289,7 @@ getSegment <- function(segment_call, segment_regex, first=TRUE, collapse=TRUE,
 #' @export
 getAllele <- function(segment_call, first=TRUE, collapse=TRUE, 
                       strip_d=TRUE, omit_nl=FALSE, sep=",") {    
-    allele_regex <- '((IG[HKL][VDJADEGMC]|TR[ABDG])[A-Z0-9\\(\\)]+[-/\\w]*[-\\*]*[\\.\\w]+)'
+    allele_regex <- '((IG[HKL]|TR[ABDG])[VDJADEGMC][A-R0-9\\(\\)]*[-/\\w]*[-\\*]*[\\.\\w]+)'
     r <- getSegment(segment_call, allele_regex, first=first, collapse=collapse, 
                     strip_d=strip_d, omit_nl=omit_nl, sep=sep)
     
@@ -294,7 +301,7 @@ getAllele <- function(segment_call, first=TRUE, collapse=TRUE,
 #' @export
 getGene <- function(segment_call, first=TRUE, collapse=TRUE, 
                     strip_d=TRUE, omit_nl=FALSE, sep=",") {
-    gene_regex <- '((IG[HKL][VDJADEGMC]|TR[ABDG])[A-Z0-9\\(\\)]+[-/\\w]*)'
+    gene_regex <- '((IG[HKL]|TR[ABDG])[VDJADEGMC][A-R0-9\\(\\)]*[-/\\w]*)'
     r <- getSegment(segment_call, gene_regex, first=first, collapse=collapse, 
                     strip_d=strip_d, omit_nl=omit_nl, sep=sep)
     
@@ -306,7 +313,7 @@ getGene <- function(segment_call, first=TRUE, collapse=TRUE,
 #' @export
 getFamily <- function(segment_call, first=TRUE, collapse=TRUE, 
                       strip_d=TRUE, omit_nl=FALSE, sep=",") {
-    family_regex <- '((IG[HKL][VDJADEGMC]|TR[ABDG])[A-Z0-9\\(\\)]+)'
+    family_regex <- '((IG[HKL]|TR[ABDG])[VDJADEGMC][A-R0-9\\(\\)]*)'
     r <- getSegment(segment_call, family_regex, first=first, collapse=collapse, 
                     strip_d=strip_d, omit_nl=omit_nl, sep=sep)
     
@@ -573,14 +580,20 @@ groupGenes <- function(data, v_call="v_call", j_call="j_call", junc_len=NULL,
                        first=FALSE) {
     # Check base input
     check <- checkColumns(data, c(v_call, j_call, junc_len))
-    if (check != TRUE) { stop(check) }
+    if (check!=TRUE) { stop("A column or some combination of columns v_call, j_call, and junc_len were not found in the data") }
     
     # Check single-cell input
     if (!is.null(cell_id)) {
-        check <- checkColumns(data, c(cell_id, locus))
+        check <- checkColumns(data, c(cell_id))
         if (check != TRUE) { stop(check) }
     }
     
+    # Check locus
+    # message locus is required
+    if (is.null(locus)) { stop("`locus`, is a required parameter.") }
+    check <- checkColumns(data, locus)
+    if (check != TRUE) { stop(check) }
+    
     # if necessary, cast select columns to character (factor not allowed later on)
     if (!is(data[[v_call]], "character")) { data[[v_call]] <- as.character(data[[v_call]]) }
     if (!is(data[[j_call]], "character")) { data[[j_call]] <- as.character(data[[j_call]]) }
@@ -631,14 +644,14 @@ groupGenes <- function(data, v_call="v_call", j_call="j_call", junc_len=NULL,
         # make a copy
         data_orig <- data; rm(data)
         
-        if (only_heavy) {
+        if(only_heavy){
             
             # use heavy chains only
             
             # Straightforward subsetting like below won't work in cases 
             #     where multiple HCs are present for a cell 
             # subset to heavy only
-            # data <- data_orig[data_orig[[locus]]=="IGH", ]
+            data <- data_orig[data_orig[[locus]] %in% c("IGH","TRB","TRD"), ]
             
             # flatten data
             cols <- c(cell_id, v_call, j_call, junc_len)
@@ -1119,7 +1132,7 @@ groupGenes <- function(data, v_call="v_call", j_call="j_call", junc_len=NULL,
             # matrix
             # important to specify rowSums from Matrix package
             # base::rowSums will NOT work
-            cell_idx <- which(Matrix::rowSums(mtx_cell_VJL[, x])>0)
+            cell_idx <- which(Matrix::rowSums(mtx_cell_VJL[, x, drop=F ])>0)
         } else {
             # vector
             cell_idx <- which(mtx_cell_VJL[, x]>0)
